import minecraft.player.InvalidCharacterParamsException;
import minecraft.player.Steve;

import java.io.*;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static java.util.Comparator.comparing;
import static java.util.stream.Collectors.groupingBy;
import static java.util.stream.Collectors.joining;

public class Main {

    public Main() throws InvalidCharacterParamsException, IOException {
    }

    public static void main(String[] args) throws IOException, InvalidCharacterParamsException, ClassNotFoundException {

        List<Steve> steveList;
        try {
            Steve steve1 = new Steve("a", 1, 1, 1, 1, 1, 1, false);
            Steve steve2 = new Steve("b", 2, 2, 1, 1, 1, 1, false);
            Steve steve3 = new Steve("c", 6, 11, 10, 1, 1, 10, false);
            Steve steve4 = new Steve("d", 4, 4, 1, 1, 1, 1, false);
            Steve steve5 = new Steve("d", 4, 4, 1, 1, 1, 1, false);

            steveList = new ArrayList<>();
            steveList.add(steve1);
            steveList.add(steve2);
            steveList.add(steve3);
            steveList.add(steve4);
            steveList.add(steve5);
//            for (int i=0; i<=10000000; i++) {
//                steveList.add(new Steve("d", 4, 4, 1, 1, 1, 1, false));
//            }

        } catch (InvalidCharacterParamsException e) {
            System.out.println("Can not create Steve objects, exiting.");
            return;
        }

        int sumSteves = steveList.stream().filter(steve -> "x".equals(steve.getName())).
                map(Steve::getHealth).reduce(0, (a, b) -> a + b);

//
//        List<Integer> steveList2 = steveList.stream().filter((steve) -> steve.getName().equals("Steve")).sorted(comparing(steve -> steve.getName())).map(Steve::getHunger).collect(Collectors.toList());
//
//        String steveString = steveList.stream().map(Steve::getName).collect(joining());
//        System.out.println(steveString);
//
//        Set<Steve> steveSet = new HashSet<>();
//        steveSet.addAll(steveList);
//
////        steveList = new ArrayList<>();
//
//        // simple parallel reduce
//        int sumHealth = steveList.parallelStream().mapToInt(Steve::getHealth).reduce(0, (a, b) -> a+b);
//        System.out.println(sumHealth);
//        int sumHealth2 = steveList.parallelStream().mapToInt(Steve::getHunger).sum();
//
////        int sumHealth3 = steveList.parallelStream().map(steve -> steve.getHealth()).reduce(Integer::sum);
//
        // Fibonacci
//        Stream.iterate(new int[]{0, 1},
//                        t -> new int[]{t[1], t[0]+t[1]})
//                .limit(50)
//                .forEach(t -> System.out.println("(" + t[0] + "," + t[1] +")"));
//
//
//        // grouping
        enum HealhLevel {LOW, NORMAL, FULL}
        Map<HealhLevel, List<Steve>> stevesByHealthLevel = steveList.stream().filter(steve -> steve.getName().equals("a")).collect(
                groupingBy(steve -> {
                    if (steve.getHealth() <= 3) return HealhLevel.LOW;
                    else if (steve.getHealth() <= 10) return HealhLevel.NORMAL;
                    else return HealhLevel.FULL;
                }));

        stevesByHealthLevel.entrySet().stream().forEach(e -> System.out.println(e.getKey() + ": " + e.getValue()));
//
//        // old map
//        Map<String, ArrayList<Steve>> steveMap = new TreeMap<>();
//        for (Steve steve : steveList) {
//            steveMap.putIfAbsent(steve.getName(), new ArrayList<>());
//            steveMap.get(steve.getName()).add(steve);
//        }
//
//        System.out.println(steveMap);


        // Streamy Uloha 1
        long countBelowHalf = steveList.stream()
                .filter(steve -> steve.getHunger() < steve.getMaxHunger() / 2)
                .count();

        System.out.println(countBelowHalf);
        // Streamy Uloha 2
        Map<Boolean, Double> avgHealthByGodMode = steveList.stream()
                .collect(Collectors.groupingBy(
                        Steve::isGodMode,
                        Collectors.averagingInt(Steve::getHealth)
                ));
        System.out.println(avgHealthByGodMode);

        // Streamy Uloha 3
        Map<String, Integer> topAttackers = steveList.stream()
                .sorted(Comparator.comparingInt(Steve::getAttackPower).reversed())
                .limit(3)
                .collect(Collectors.toMap(
                        Steve::getName,
                        Steve::getAttackPower
                ));

        System.out.println(topAttackers);

        // Streamy Uloha 4
        List<Steve> healthyHungerSteves = steveList.stream()
                .filter(steve -> steve.getHealth() > steve.getMaxHealth() * 0.5)
                .filter(steve -> steve.getHunger() < steve.getMaxHunger() * 0.4)
                .sorted(Comparator.comparing(Steve::getName))
                .toList();
        System.out.println(healthyHungerSteves);

        // Serializacia Uloha 1
        System.out.println(steveList);
        ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream("steves_collection.ser"));
        oos.writeObject(steveList);
        oos.close();

        // Serializacia Uloha 2
        ObjectInputStream ois = new ObjectInputStream(new FileInputStream("steves_collection.ser"));
        List<Steve> deserialized = (List<Steve>) ois.readObject();
        ois.close();

        deserialized.forEach(System.out::println);
    }
}