package minecraft.player;

import minecraft.item.Food;
import minecraft.item.Item;

import java.io.Serial;
import java.io.Serializable;
import java.util.Objects;

/**
 * @author Juraj Petrik
 * @version 0.8
 */
public class Steve extends Character implements Comparable<Steve>, Serializable {

    private java.util.UUID uuid = java.util.UUID.randomUUID();


    private int hunger;
    private int maxHunger;
    private boolean godMode; // nesmrtelnost, zabija na jednu ranu (pokial nie je godMode)

    private Item items;

    /**
     *
     * @param name
     * @param health
     * @param maxHealth
     * @param attackPower
     * @param defensePower
     * @param hunger
     * @param maxHunger
     * @param godMode
     * @throws InvalidCharacterParamsException
     */
    public Steve(String name, int health, int maxHealth, int attackPower, int defensePower, int hunger, int maxHunger, boolean godMode) throws InvalidCharacterParamsException {
        super(name, health, maxHealth, attackPower, defensePower);

        if (hunger <= 0 || maxHunger <= 0)
            throw new InvalidCharacterParamsException("Cannot be negative or zero!");

        if (hunger > maxHunger) {
            throw new InvalidCharacterParamsException("Hunger can not be larger than maxHunger");
        }

        this.setHunger(hunger);
        this.setMaxHunger(maxHunger);
        this.setGodMode(godMode);
    }

    /**
     * Consumes Food if there is any food to consume
     * @param food
     */
    public void consume(Food food) {

        if (food.getAmount() > 0) {
            setHealth(Math.min(getHealth() + food.getHeal(), getMaxHealth()));
            setHunger(Math.min(getHunger() + food.getFeed(), getMaxHunger()));

            food.setAmount(food.getAmount() - 1);

            System.out.println(getName() + " sa najedol, hunger je " + getHunger() + " health je " + getHealth());
        }
    }

    // toto nie je idealna implementacia z hladiska znovupouzitelnosti, ale na ukazku override je to ok
    public void attack(NPC who) {
        int realAttackPower = godMode ? who.getMaxHealth() : Math.max(0, getAttackPower() - who.getDefensePower());
        who.setHealth(who.getHealth() - realAttackPower);

        System.out.println(getName() + " zautotocil na " + who.getName() + " damage je " + realAttackPower);
    }

    /**
     *
     * @param who If Creeper has godmode than it is instant kill
     */
    public void attack(Creeper who) {
        int realAttackPower = godMode ? who.getMaxHealth() : Math.max(0, getAttackPower() - who.getDefensePower());

        who.setHealth(who.getHealth() - realAttackPower);
        who.setBombTimer(1.5);

        System.out.println(getName() + " zautotocil na " + who.getName() + " damage je " + realAttackPower);
    }

    public void attack(Steve who) {
        if (who.isGodMode())
            return;

        int realAttackPower = godMode ? who.getMaxHealth() : Math.max(0, getAttackPower() - who.getDefensePower());

        who.setHealth(who.getHealth() - realAttackPower);
        System.out.println(getName() + " zautotocil na " + who.getName() + " damage je " + realAttackPower);
    }


    public int getHunger() {
        return hunger;
    }

    public void setHunger(int hunger) {
        this.hunger = hunger;
    }

    public int getMaxHunger() {
        return maxHunger;
    }

    public void setMaxHunger(int maxHunger) {
        this.maxHunger = maxHunger;
    }

    public boolean isGodMode() {
        return godMode;
    }

    public void setGodMode(boolean godMode) {
        this.godMode = godMode;
    }

    @Override
    public int compareTo(Steve o) {
        return o.getName().compareTo(this.getName());
    }

    @Override
    public boolean equals(Object object) {
        if (this == object) return true;
        if (!(object instanceof Steve steve)) return false;

        // Objects.equals is handy - it is null safe!
        // tu je chyba v implementacii (uz nie je :)) /getMaxHealth() == steve.getMaxHealth()/
        return Objects.equals(getName(), steve.getName()) && getHealth() == steve.getHealth() && getMaxHealth() == steve.getMaxHealth()
                && getAttackPower() == steve.getAttackPower() && getDefensePower() == steve.getDefensePower() &&
                getHunger() == steve.getHunger() && getMaxHunger() == steve.getMaxHunger() && isGodMode() == steve.isGodMode();
    }
    @Override
    public int hashCode() {
        // one of many implementations
        return Objects.hash(getName(), getHealth(), getMaxHealth(), getAttackPower(), getDefensePower(), getHunger(),
                getMaxHunger(), isGodMode());
    }
}
